@@ -13,8 +13,9 @@ Readonly my $_360_DEGREES       => 360;
 Readonly my $FULLPAGE_OCR_SCALE => 0.8;
 my $SPACE = q{ };
 my $EMPTY = q{};
+my $device;
 
-our $VERSION = '2.2.2';
+our $VERSION = '2.5.1';
 
 use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
     'zoom-changed' => {
@@ -36,6 +37,10 @@ use Glib::Object::Subclass GooCanvas2::Canvas::, signals => {
 sub INIT_INSTANCE {
     my $self = shift;
 
+    my $display = Gtk3::Gdk::Display::get_default;
+    my $manager = $display->get_device_manager;
+    $device = $manager->get_client_pointer;
+
     # Set up the canvas
     $self->signal_connect( 'button-press-event'   => \&_button_pressed );
     $self->signal_connect( 'button-release-event' => \&_button_released );
@@ -266,9 +271,15 @@ sub _boxed_text {
 
         # clicking text box produces a dialog to edit the text
         if ($edit_callback) {
-            $text->signal_connect( 'button-press-event' =>
-                  sub { $root->get_parent->{dragging} = FALSE } );
-            $text->signal_connect( 'button-press-event' => $edit_callback );
+            $text->signal_connect(
+                'button-press-event' => sub {
+                    my ( $widget, $target, $event ) = @_;
+                    if ( $event->button == 1 ) {
+                        $root->get_parent->{dragging} = FALSE;
+                        $edit_callback->( $widget, $target, $event );
+                    }
+                }
+            );
         }
     }
     if ( $box->{contents} ) {
@@ -495,13 +506,18 @@ sub _clamp_direction {
 sub _button_pressed {
     my ( $self, $event ) = @_;
 
-    # left mouse button
-    if ( $event->button != 1 ) { return FALSE }
+    # middle mouse button
+    if ( $event->button == 2 ) {
 
-    $self->{drag_start} = { x => $event->x, y => $event->y };
-    $self->{dragging} = TRUE;
+        # Using the root window x,y position for dragging the canvas, as the
+        # values returned by $event->x and y cause a bouncing effect, and
+        # only the value since the last event is required.
+        my ( $screen, $x, $y ) = $device->get_position;
+        $self->{drag_start} = { x => $x, y => $y };
+        $self->{dragging} = TRUE;
 
-    #    $self->update_cursor( $event->x, $event->y );
+        #    $self->update_cursor( $event->x, $event->y );
+    }
 
     # allow the event to propagate in case the user was clicking on text to edit
     return;
@@ -509,7 +525,7 @@ sub _button_pressed {
 
 sub _button_released {
     my ( $self, $event ) = @_;
-    $self->{dragging} = FALSE;
+    if ( $event->button == 2 ) { $self->{dragging} = FALSE }
 
     #    $self->update_cursor( $event->x, $event->y );
     return;
@@ -521,12 +537,10 @@ sub _motion {
 
     my $offset = $self->get_offset;
     my $zoom   = $self->get_scale;
-    my $offset_x =
-      $offset->{x} + ( $event->x - $self->{drag_start}{x} ) / $zoom;
-    my $offset_y =
-      $offset->{y} + ( $event->y - $self->{drag_start}{y} ) / $zoom;
-    ( $self->{drag_start}{x}, $self->{drag_start}{y} ) =
-      ( $event->x, $event->y );
+    my ( $screen, $x, $y ) = $device->get_position;
+    my $offset_x = $offset->{x} + ( $x - $self->{drag_start}{x} ) / $zoom;
+    my $offset_y = $offset->{y} + ( $y - $self->{drag_start}{y} ) / $zoom;
+    ( $self->{drag_start}{x}, $self->{drag_start}{y} ) = ( $x, $y );
     $self->set_offset( $offset_x, $offset_y );
     return;
 }
