@@ -17,7 +17,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '2.2.2';
+    $VERSION = '2.5.1';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -128,6 +128,18 @@ sub _pre_200 {
     return;
 }
 
+sub _pre_223 {
+    my ( $version, $SETTING ) = @_;
+    if ( version->parse($version) < version->parse('2.2.3') ) {
+        if ( defined $SETTING->{'date offset'} ) {
+            $SETTING->{'datetime offset'} =
+              [ $SETTING->{'date offset'}, 0, 0, 0 ];
+            delete $SETTING->{'date offset'};
+        }
+    }
+    return;
+}
+
 sub read_config {
     my ( $filename, $logger ) = @_;
     my ( %SETTING, $conf );
@@ -201,6 +213,8 @@ sub read_config {
 
     _pre_200( $version, \%SETTING );
 
+    _pre_223( $version, \%SETTING );
+
     $logger->debug( Dumper( \%SETTING ) );
     return %SETTING;
 }
@@ -234,6 +248,8 @@ sub add_defaults {
         'thumb panel'                       => 100,
         scan_window_width                   => undef,
         scan_window_height                  => undef,
+        message_window_width                => 600,
+        message_window_height               => 200,
         TMPDIR                              => undef,
         'Page range'                        => 'all',
         version                             => undef,
@@ -257,7 +273,7 @@ sub add_defaults {
         set_timestamp                       => TRUE,
         use_time                            => FALSE,
         use_timezone                        => TRUE,
-        'date offset'                       => 0,
+        'datetime offset'                   => [ 0, 0, 0, 0 ],
         'pdf compression'                   => 'auto',
         'tiff compression'                  => undef,
         'pdf font'                          => undef,
@@ -291,6 +307,7 @@ sub add_defaults {
         'threshold tool'                    => 80,
         'Blank threshold' => 0.005,    # Blank page standard deviation threshold
         'Dark threshold'  => 0.12,     # Dark page mean threshold
+        to_png            => TRUE,
         'OCR on scan'     => TRUE,
         'ocr engine'   => 'tesseract',
         'ocr language' => undef,
