@@ -43,10 +43,12 @@ BEGIN {
 # from http://gtk2-perl.sourceforge.net/doc/subclassing_widgets_in_perl.html
 use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     'new-scan' => {
-        param_types => [ 'Glib::String', 'Glib::UInt' ], # filename, page number
+        param_types =>
+          [ 'Glib::String', 'Glib::UInt', 'Glib::Float', 'Glib::Float' ]
+        ,    # filename, page number, x-resolution, y-resolution
     },
     'changed-device' => {
-        param_types => ['Glib::String'],                 # device name
+        param_types => ['Glib::String'],    # device name
     },
     'changed-device-list' => {
         param_types => ['Glib::Scalar'],    # array of hashes with device info
@@ -267,7 +269,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.2.2';
+our $VERSION = '2.5.1';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -523,8 +525,8 @@ sub INIT_INSTANCE {
     my $vboxsp = Gtk3::VBox->new;
     $vboxsp->set_border_width($border_width);
     $framesp->add($vboxsp);
-    $self->{combobsp} = Gscan2pdf::ComboBoxText->new;
-    $self->{combobsp}->signal_connect(
+    $self->{combobsp}                = Gscan2pdf::ComboBoxText->new;
+    $self->{combobsp_changed_signal} = $self->{combobsp}->signal_connect(
         changed => sub {
             $self->{num_reloads} = 0;    # num-reloads is read-only
             $self->set( 'profile', $self->{combobsp}->get_active_text );
@@ -539,6 +541,11 @@ sub INIT_INSTANCE {
     $vbutton->signal_connect( clicked => \&_save_profile_callback, $self );
     $hboxsp->pack_start( $vbutton, TRUE, TRUE, 0 );
 
+    # Edit button
+    my $ebutton = Gtk3::Button->new_from_stock('gtk-edit');
+    $ebutton->signal_connect( clicked => \&_edit_profile_callback, $self );
+    $hboxsp->pack_start( $ebutton, FALSE, FALSE, 0 );
+
     # Delete button
     my $dbutton = Gtk3::Button->new_from_stock('gtk-delete');
     $dbutton->signal_connect(
@@ -671,6 +678,84 @@ sub _save_profile_callback {
     return;
 }
 
+# Clone current profile, display list of options, allowing user to delete those
+# not required, and then to cancel or accept the changes.
+
+sub _edit_profile_callback {
+    my ( $widget, $parent ) = @_;
+    my $name = $parent->get('profile');
+    if ( not defined $name or $name eq $EMPTY ) { return }
+    my $dialog = Gtk3::Dialog->new(
+        sprintf( __('Editing scan profile "%s"'), $name ), $parent,
+        'destroy-with-parent',
+        'gtk-ok'     => 'ok',
+        'gtk-cancel' => 'cancel'
+    );
+    my $label = Gtk3::Label->new( sprintf __('Scan profile "%s"'), $name );
+    $dialog->get_content_area->pack_start( $label, TRUE, TRUE, 0 );
+
+    # Clone so that we can cancel the changes, if necessary
+    my $profile = dclone( $parent->{profiles}{$name} );
+    _build_profile_table( $profile, $parent->get('available-scan-options'),
+        $dialog->get_content_area );
+    $dialog->set_default_response('ok');
+    $dialog->show_all;
+
+    # save the profile and reload
+    if ( $dialog->run eq 'ok' ) {
+        $parent->{profiles}{$name} = $profile;
+
+        # unset profile to allow us to set it again on reload
+        $parent->{profile} = undef;
+
+        # emit signal to update settings
+        $parent->signal_emit( 'added-profile', $name,
+            $parent->{profiles}{$name} );
+        my $signal;
+        $signal = $parent->signal_connect(
+            'reloaded-scan-options' => sub {
+                $parent->signal_handler_disconnect($signal);
+                $parent->set_profile($name);
+            }
+        );
+        $parent->scan_options( $parent->get('device') );
+    }
+    $dialog->destroy;
+    return;
+}
+
+sub _build_profile_table {
+    my ( $profile, $options, $vbox ) = @_;
+
+    # listbox to align widgets
+    my $listbox = Gtk3::ListBox->new;
+    $listbox->set_selection_mode('none');
+    $vbox->pack_start( $listbox, TRUE, TRUE, 0 );
+    my $iter = $profile->each_backend_option;
+    while ( my $i = $iter->() ) {
+        my ( $name, $val ) = $profile->get_backend_option_by_index($i);
+        my $opt   = $options->by_name($name);
+        my $row   = Gtk3::ListBoxRow->new;
+        my $hbox  = Gtk3::HBox->new;
+        my $label = Gtk3::Label->new( $d_sane->get( $opt->{title} ) );
+        $hbox->pack_start( $label, FALSE, TRUE, 0 );
+        my $button = Gtk3::Button->new_from_stock('gtk-delete');
+        $hbox->pack_end( $button, FALSE, FALSE, 0 );
+        $button->signal_connect(
+            clicked => sub {
+                $logger->debug("removing option '$name' from profile");
+                $profile->remove_backend_option_by_index($i);
+                $listbox->destroy;
+                _build_profile_table( $profile, $options, $vbox );
+            }
+        );
+        $row->add($hbox);
+        $listbox->add($row);
+    }
+    $vbox->show_all;
+    return;
+}
+
 sub SET_PROPERTY {
     my ( $self, $pspec, $newval ) = @_;
     my $name   = $pspec->get_name;
@@ -1248,7 +1333,7 @@ sub update_options {
     }
 
     # Reapply current options to ensure the same values are still set.
-    $self->set_current_scan_options($current_scan_options);
+    $self->add_current_scan_options($current_scan_options);
 
     # In case the geometry values have changed,
     # update the available paper formats
@@ -1456,7 +1541,7 @@ sub set_paper {
 
     # Don't trigger the changed-paper signal
     # until we have finished setting the profile
-    $self->set_current_scan_options($paper_profile);
+    $self->add_current_scan_options($paper_profile);
     return;
 }
 
@@ -1642,7 +1727,12 @@ sub edit_paper {
 sub save_current_profile {
     my ( $self, $name ) = @_;
     $self->add_profile( $name, $self->{current_scan_options} );
+
+    # Block signal or else we fire another round of profile loads
+    $self->{combobsp}->signal_handler_block( $self->{combobsp_changed_signal} );
     $self->{combobsp}->set_active( $self->{combobsp}->get_num_rows - 1 );
+    $self->{combobsp}
+      ->signal_handler_unblock( $self->{combobsp_changed_signal} );
     return;
 }
 
@@ -2072,6 +2162,34 @@ sub set_current_scan_options {
         return;
     }
 
+    # reload to get defaults before applying profile
+    my $signal;
+    $self->{current_scan_options} =
+      Gscan2pdf::Scanner::Profile->new_from_data( dclone($profile) );
+    $signal = $self->signal_connect(
+        'reloaded-scan-options' => sub {
+            $self->signal_handler_disconnect($signal);
+            $self->add_current_scan_options($profile);
+        }
+    );
+    $self->scan_options( $self->get('device') );
+    return;
+}
+
+# Apply options referenced by hashref without resetting existing options
+
+sub add_current_scan_options {
+    my ( $self, $profile ) = @_;
+    if ( not defined $profile ) {
+        $logger->error('Cannot add undefined profile');
+        return;
+    }
+    elsif ( ref($profile) ne 'Gscan2pdf::Scanner::Profile' ) {
+        $logger->error(
+            ref($profile) . ' is not a Gscan2pdf::Scanner::Profile object' );
+        return;
+    }
+
     # First clone the profile, as otherwise it would be self-modifying
     my $clone = dclone($profile);
 
@@ -2144,6 +2262,10 @@ sub _set_option_profile {
     else {
 
         # Having set all backend options, set the frontend options
+        # Set paper formats first to make sure that any paper required is
+        # available
+        $self->set_paper_formats( $self->{paper_formats} );
+
         my $iter = $profile->each_frontend_option;
         while ( my $key = $iter->() ) {
             $self->set( $key, $profile->get_frontend_option($key) );
@@ -2209,6 +2331,54 @@ sub make_progress_string {
     return sprintf __('Scanning page %d'), $i;
 }
 
+sub get_xy_resolution {
+    my ($self) = @_;
+    my $options = $self->get('available-scan-options');
+    if ( not defined $options ) { return }
+    my $resolution = $options->by_name(SANE_NAME_SCAN_RESOLUTION);
+    my $x          = $options->by_name('x-resolution');
+    my $y          = $options->by_name('y-resolution');
+    if ( defined $resolution ) { $resolution = $resolution->{val} }
+    if ( defined $x )          { $x          = $x->{val} }
+    if ( defined $y )          { $y          = $y->{val} }
+
+    # Potentially, a scanner could offer all three options, but then unset
+    # resolution once the other two have been set.
+    if ( defined $resolution ) {
+
+        # The resolution option, plus one of the other two, is defined.
+        # Most sensibly, we should look at the order they were set.
+        # However, if none of them are in current-scan-options, they still have
+        # their default setting, and which of those gets priority is certainly
+        # scanner specific.
+        if ( defined $x or defined $y ) {
+            my $current_scan_options = $self->get('current-scan-options');
+            my $iter = $current_scan_options->each_backend_option;
+            while ( my $i = $iter->() ) {
+                my ( $name, $val ) =
+                  $current_scan_options->get_backend_option_by_index($i);
+                if ( $name eq SANE_NAME_SCAN_RESOLUTION ) {
+                    $x = $val;
+                    $y = $val;
+                }
+                elsif ( $name eq 'x-resolution' ) {
+                    $x = $val;
+                }
+                elsif ( $name eq 'y-resolution' ) {
+                    $y = $val;
+                }
+            }
+        }
+        else {
+            return $resolution, $resolution;
+        }
+    }
+
+    if ( not defined $x ) { $x = $Gscan2pdf::Document::POINTS_PER_INCH }
+    if ( not defined $y ) { $y = $Gscan2pdf::Document::POINTS_PER_INCH }
+    return $x, $y;
+}
+
 1;
 
 __END__
